/*
Copyright (c) 2019-2024 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package disks

import (
	"context"
	"strconv"

	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudapi/disks"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/dc"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
)

func resourceDiskReplicationCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	diskId := uint64(d.Get("disk_id").(int))

	log.Debugf("resourceDiskReplicationCreate: called for disk with ID: %d", diskId)

	c := m.(*controller.ControllerCfg)

	err := existDiskID(ctx, diskId, m)
	if err != nil {
		return diag.FromErr(err)
	}

	reqCreate := disks.ReplicateRequest{
		DiskID:   diskId,
		Name:     d.Get("disk_name").(string),
		SepID:    uint64(d.Get("sep_id").(int)),
		PoolName: d.Get("pool_name").(string),
	}

	diskReplicaId, err := c.CloudAPI().Disks().Replicate(ctx, reqCreate)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	d.SetId(strconv.FormatUint(diskReplicaId, 10))
	d.Set("replica_disk_id", diskReplicaId)

	log.Debugf("resourceDiskReplicationCreate: create replica complete for disk with ID: %d", diskId)

	warnings := dc.Warnings{}

	if start, ok := d.GetOk("start"); ok && !start.(bool) {
		log.Debugf("resourceDiskReplicationCreate: replication between disk with ID: %d and replica with ID: %d, try to stop", diskId, diskReplicaId)
		reqStop := disks.ReplicationStopRequest{
			DiskID: diskId,
		}
		_, err = c.CloudAPI().Disks().ReplicationStop(ctx, reqStop)
		if err != nil {
			warnings.Add(err)
		}
		log.Debugf("resourceDiskReplicationCreate: replication between disk with ID: %d and replica with ID: %d, stoped", diskId, diskReplicaId)
	}
	return append(resourceDiskReplicationRead(ctx, d, m), warnings.Get()...)
}

func resourceDiskReplicationRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceDiskReplicationRead: called for disk with ID: %s", d.Id())
	c := m.(*controller.ControllerCfg)

	req := disks.ReplicationStatusRequest{
		DiskID: uint64(d.Get("disk_id").(int)),
	}

	status, err := c.CloudAPI().Disks().ReplicationStatus(ctx, req)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	diskReplica, err := utilityDiskReplicaCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	flattenDiskReplica(d, diskReplica, status)

	log.Debugf("resourceDiskReplicationRead: read complete for disk with ID: %s", d.Id())
	return nil
}

func resourceDiskReplicationUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	diskId := uint64(d.Get("disk_id").(int))
	log.Debugf("resourceDiskReplicationUpdate: called for disk with ID: %d", diskId)

	err := existDiskID(ctx, diskId, m)
	if err != nil {
		return diag.FromErr(err)
	}

	if d.HasChange("start") {
		if err := utilityDiskReplicationUpdateStartStop(ctx, d, m); err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("pause") {
		if err := utilityDiskReplicationUpdatePause(ctx, d, m); err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("reverse") {
		if err := utilityDiskReplicationUpdateReverse(ctx, d, m); err != nil {
			return diag.FromErr(err)
		}
	}

	log.Debugf("resourceDiskReplicationUpdate: read complete for disk with ID: %d", diskId)
	return resourceDiskReplicationRead(ctx, d, m)
}

func resourceDiskReplicationDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	diskId := uint64(d.Get("disk_id").(int))
	log.Debugf("resourceDiskReplicationDelete: called for disk with ID: %d", diskId)

	disk, err := utilityDiskReplicaCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)

	if d.Get("start").(bool) {
		reqStop := disks.ReplicationStopRequest{
			DiskID: uint64(d.Get("disk_id").(int)),
		}

		log.Debugf("resourceDiskReplicationDelete: stop replication for disk with ID: %d", diskId)
		_, err = c.CloudAPI().Disks().ReplicationStop(ctx, reqStop)
		if err != nil {
			return diag.FromErr(err)
		}
		log.Debugf("resourceDiskReplicationDelete: stop replication for disk with ID: %d, complete", diskId)
	}

	reqDelete := disks.DeleteRequest{
		DiskID:      disk.ID,
		Detach:      d.Get("detach").(bool),
		Permanently: d.Get("permanently").(bool),
	}

	log.Debugf("resourceDiskReplicationDelete: delete disk replica for disk with ID: %d", diskId)
	_, err = c.CloudAPI().Disks().Delete(ctx, reqDelete)
	if err != nil {
		return diag.FromErr(err)
	}
	log.Debugf("resourceDiskReplicationDelete: delete disk replica for disk with ID: %d, complete", diskId)

	d.SetId("")

	return nil
}

func resourceDiskReplicationSchemaMake() map[string]*schema.Schema {
	rets := map[string]*schema.Schema{
		"disk_id": {
			Type:        schema.TypeInt,
			Required:    true,
			Description: "Id of primary disk",
		},
		"disk_name": {
			Type:        schema.TypeString,
			Required:    true,
			Description: "Name of disk replica",
		},
		"sep_id": {
			Type:        schema.TypeInt,
			Required:    true,
			Description: "Storage endpoint provider ID to create disk replica",
		},
		"pool_name": {
			Type:        schema.TypeString,
			Required:    true,
			Description: "Pool for disk location",
		},
		"pause": {
			Type:        schema.TypeBool,
			Optional:    true,
			Description: "Resume replication",
		},
		"reverse": {
			Type:        schema.TypeBool,
			Optional:    true,
			Default:     false,
			Description: "Reverse replication",
		},
		"start": {
			Type:        schema.TypeBool,
			Optional:    true,
			Default:     true,
			Description: "Start/Stop replication",
		},
		"detach": {
			Type:        schema.TypeBool,
			Optional:    true,
			Default:     false,
			Description: "Detach disk from machine first",
		},
		"permanently": {
			Type:        schema.TypeBool,
			Optional:    true,
			Default:     false,
			Description: "Delete disk permanently",
		},
		"replica_disk_id": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Id of replica disk",
		},
		"status_replication": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Status of replication",
		},
		"account_id": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "The unique ID of the subscriber-owner of the disk",
		},
		"account_name": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "The name of the subscriber '(account') to whom this disk belongs",
		},
		"acl": {
			Type:     schema.TypeString,
			Computed: true,
		},
		// "boot_partition": {
		// 	Type:        schema.TypeInt,
		// 	Computed:    true,
		// 	Description: "Number of disk partitions",
		// },
		"computes": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Resource{
				Schema: map[string]*schema.Schema{
					"compute_id": {
						Type:     schema.TypeString,
						Computed: true,
					},
					"compute_name": {
						Type:     schema.TypeString,
						Computed: true,
					},
				},
			},
		},
		"created_time": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Created time",
		},
		"deleted_time": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Deleted time",
		},
		"desc": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Description of disk",
		},
		"destruction_time": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Time of final deletion",
		},
		"devicename": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Name of the device",
		},
		// "disk_path": {
		// 	Type:        schema.TypeString,
		// 	Computed:    true,
		// 	Description: "Disk path",
		// },
		"gid": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "ID of the grid (platform)",
		},
		// "guid": {
		// 	Type:        schema.TypeInt,
		// 	Computed:    true,
		// 	Description: "Disk ID on the storage side",
		// },
		"image_id": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Image ID",
		},
		"images": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Schema{
				Type: schema.TypeString,
			},
			Description: "IDs of images using the disk",
		},
		"iotune": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Resource{
				Schema: map[string]*schema.Schema{
					"read_bytes_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Number of bytes to read per second",
					},
					"read_bytes_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum number of bytes to read",
					},
					"read_iops_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Number of io read operations per second",
					},
					"read_iops_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum number of io read operations",
					},
					"size_iops_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Size of io operations",
					},
					"total_bytes_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Total size bytes per second",
					},
					"total_bytes_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum total size of bytes per second",
					},
					"total_iops_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Total number of io operations per second",
					},
					"total_iops_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum total number of io operations per second",
					},
					"write_bytes_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Number of bytes to write per second",
					},
					"write_bytes_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum number of bytes to write per second",
					},
					"write_iops_sec": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Number of write operations per second",
					},
					"write_iops_sec_max": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Maximum number of write operations per second",
					},
				},
			},
		},
		// "iqn": {
		// 	Type:        schema.TypeString,
		// 	Computed:    true,
		// 	Description: "Disk IQN",
		// },
		// "login": {
		// 	Type:        schema.TypeString,
		// 	Computed:    true,
		// 	Description: "Login to access the disk",
		// },
		// "milestones": {
		// 	Type:        schema.TypeInt,
		// 	Computed:    true,
		// 	Description: "Milestones",
		// },
		"order": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Disk order",
		},
		"params": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Disk params",
		},
		"parent_id": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "ID of the parent disk",
		},
		// "passwd": {
		// 	Type:        schema.TypeString,
		// 	Computed:    true,
		// 	Description: "Password to access the disk",
		// },
		"pci_slot": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "ID of the pci slot to which the disk is connected",
		},
		"present_to": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Schema{
				Type: schema.TypeInt,
			},
		},
		// "purge_attempts": {
		// 	Type:        schema.TypeInt,
		// 	Computed:    true,
		// 	Description: "Number of deletion attempts",
		// },
		"purge_time": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Time of the last deletion attempt",
		},
		"replication": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Resource{
				Schema: map[string]*schema.Schema{
					"disk_id": {
						Type:     schema.TypeInt,
						Computed: true,
					},
					"pool_id": {
						Type:     schema.TypeString,
						Computed: true,
					},
					"role": {
						Type:     schema.TypeString,
						Computed: true,
					},
					"self_volume_id": {
						Type:     schema.TypeString,
						Computed: true,
					},
					"storage_id": {
						Type:     schema.TypeString,
						Computed: true,
					},
					"volume_id": {
						Type:     schema.TypeString,
						Computed: true,
					},
				},
			},
			Description: "Replication status",
		},
		// "reality_device_number": {
		// 	Type:        schema.TypeInt,
		// 	Computed:    true,
		// 	Description: "Reality device number",
		// },
		// "reference_id": {
		// 	Type:        schema.TypeString,
		// 	Computed:    true,
		// 	Description: "ID of the reference to the disk",
		// },
		"res_id": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Resource ID",
		},
		"res_name": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Name of the resource",
		},
		"role": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Disk role",
		},
		"sep_type": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Type SEP. Defines the type of storage system and contains one of the values set in the cloud platform",
		},
		"shareable": {
			Type:     schema.TypeBool,
			Computed: true,
		},
		"size_max": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Size in GB",
		},
		"size_used": {
			Type:        schema.TypeFloat,
			Computed:    true,
			Description: "Number of used space, in GB",
		},
		"snapshots": {
			Type:     schema.TypeList,
			Computed: true,
			Elem: &schema.Resource{
				Schema: map[string]*schema.Schema{
					"guid": {
						Type:        schema.TypeString,
						Computed:    true,
						Description: "ID of the snapshot",
					},
					"label": {
						Type:        schema.TypeString,
						Computed:    true,
						Description: "Name of the snapshot",
					},
					"res_id": {
						Type:        schema.TypeString,
						Computed:    true,
						Description: "Reference to the snapshot",
					},
					"snap_set_guid": {
						Type:        schema.TypeString,
						Computed:    true,
						Description: "The set snapshot ID",
					},
					"snap_set_time": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "The set time of the snapshot",
					},
					"timestamp": {
						Type:        schema.TypeInt,
						Computed:    true,
						Description: "Snapshot time",
					},
				},
			},
		},
		"status": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Disk status",
		},
		"tech_status": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "Technical status of the disk",
		},
		"type": {
			Type:        schema.TypeString,
			Computed:    true,
			Description: "The type of disk in terms of its role in compute: 'B=Boot, D=Data, T=Temp'",
		},
		"vmid": {
			Type:        schema.TypeInt,
			Computed:    true,
			Description: "Virtual Machine ID (Deprecated)",
		},
	}

	return rets
}

func ResourceDiskReplication() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceDiskReplicationCreate,
		ReadContext:   resourceDiskReplicationRead,
		UpdateContext: resourceDiskReplicationUpdate,
		DeleteContext: resourceDiskReplicationDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: resourceDiskReplicationSchemaMake(),
	}
}
