/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package extnet

import (
	"context"
	"fmt"
	"strconv"
	"strings"

	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/extnet"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
)

func utilityDataStaticRouteCheckPresence(ctx context.Context, d *schema.ResourceData, m interface{}) (*extnet.ItemRoutes, error) {
	c := m.(*controller.ControllerCfg)
	req := extnet.StaticRouteListRequest{}
	var routeId uint64

	if d.Id() != "" {
		arr := strings.Split(d.Id(), "#")
		if len(arr) != 2 {
			return nil, fmt.Errorf("broken state id")
		}

		req.ExtNetID, _ = strconv.ParseUint(arr[0], 10, 64)
		routeId, _ = strconv.ParseUint(arr[1], 10, 64)
	} else {
		req.ExtNetID = uint64(d.Get("extnet_id").(int))
		routeId = uint64(d.Get("route_id").(int))
	}

	log.Debugf("utilityStaticRouteCheckPresence, extnet id: %v", req.ExtNetID)
	staticRouteList, err := c.CloudBroker().ExtNet().StaticRouteList(ctx, req)
	if err != nil {
		return nil, err
	}

	log.Debugf("utilityStaticRouteCheckPresence: ROUTE ID %v", routeId)

	staticRoute := extnet.ItemRoutes{}
	for _, route := range staticRouteList.Data {
		if routeId == route.ID {
			staticRoute = route
			return &staticRoute, nil
		}
	}

	return nil, fmt.Errorf("static route not found")
}

func getStaticRouteData(ctx context.Context, d *schema.ResourceData, m interface{}) (*extnet.ItemRoutes, error) {
	c := m.(*controller.ControllerCfg)
	req := extnet.StaticRouteListRequest{}
	req.ExtNetID = uint64(d.Get("extnet_id").(int))

	staticRouteList, err := c.CloudBroker().ExtNet().StaticRouteList(ctx, req)
	if err != nil {
		return nil, err
	}

	destination := d.Get("destination").(string)
	gateway := d.Get("gateway").(string)

	staticRoute := extnet.ItemRoutes{}
	for _, route := range staticRouteList.Data {
		if destination == route.Destination && gateway == route.Gateway {
			staticRoute = route
			return &staticRoute, nil
		}
	}

	return nil, fmt.Errorf("static route not found")
}

func utilityStaticRouteComputeIDsUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	staticRouteData, err := utilityDataStaticRouteCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return err
	}

	oldSet, newSet := d.GetChange("compute_ids")

	deletedComputeIDs := (oldSet.(*schema.Set).Difference(newSet.(*schema.Set))).List()
	deletedIds := make([]uint64, 0, len(deletedComputeIDs))
	if len(deletedComputeIDs) > 0 {
		for _, computeIdInterface := range deletedComputeIDs {
			deletedIds = append(deletedIds, uint64(computeIdInterface.(int)))
		}

		req := extnet.StaticRouteAccessRevokeRequest{
			ExtNetID:   uint64(d.Get("extnet_id").(int)),
			RouteId:    staticRouteData.ID,
			ComputeIds: deletedIds,
		}

		_, err := c.CloudBroker().ExtNet().StaticRouteAccessRevoke(ctx, req)
		if err != nil {
			return err
		}
	}

	addedComputeIDs := (newSet.(*schema.Set).Difference(oldSet.(*schema.Set))).List()
	addedIds := make([]uint64, 0, len(addedComputeIDs))
	if len(addedComputeIDs) > 0 {
		for _, computeIdInterface := range addedComputeIDs {
			addedIds = append(addedIds, uint64(computeIdInterface.(int)))
		}
		req := extnet.StaticRouteAccessGrantRequest{
			ExtNetID:   uint64(d.Get("extnet_id").(int)),
			RouteId:    staticRouteData.ID,
			ComputeIds: addedIds,
		}

		_, err := c.CloudBroker().ExtNet().StaticRouteAccessGrant(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}
