/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Sergey Kisil, <svkisil@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package vins

import (
	"context"
	"fmt"
	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/dc"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/ic"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func checkParamsExistenceCreateInAcc(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) diag.Diagnostics {
	var errs []error

	accountId := uint64(d.Get("account_id").(int))

	if err := ic.ExistAccount(ctx, accountId, c); err != nil {
		errs = append(errs, err)
	}

	if gid, ok := d.GetOk("gid"); ok {
		if err := ic.ExistGID(ctx, uint64(gid.(int)), c); err != nil {
			errs = append(errs, err)
		}
	}

	return dc.ErrorsToDiagnostics(errs)
}

func checkParamsExistenceCreateInRG(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) diag.Diagnostics {
	var errs []error

	rgId := uint64(d.Get("rg_id").(int))

	if err := ic.ExistGID(ctx, rgId, c); err != nil {
		errs = append(errs, err)
	}

	extNetID, ok := d.GetOk("ext_net_id")
	if ok {
		if err := ic.ExistExtNetInVins(ctx, extNetID.(int), c); err != nil {
			errs = append(errs, err)
		}
	}

	if extIP, ok := d.GetOk("ext_ip"); ok {
		if extNetID.(int) < 0 {
			errs = append(errs, fmt.Errorf("external IP is related only for extNetId >= 0: ext_ip %d, extNetId %d", extIP, extNetID))
		}
	}

	return nil
}

func checkParamsExistenceUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) diag.Diagnostics {
	var errs []error

	if rgId, ok := d.GetOk("rg_id"); ok {
		err := ic.ExistRG(ctx, uint64(rgId.(int)), c)
		if err != nil {
			errs = append(errs, err)
		}
	}

	if extNetId, ok := d.GetOk("ext_net_id"); ok {
		err := ic.ExistExtNetInVins(ctx, extNetId.(int), c)
		if err != nil {
			errs = append(errs, err)
		}
	}

	if accountId, ok := d.GetOk("account_id"); ok {
		err := ic.ExistAccount(ctx, uint64(accountId.(int)), c)
		if err != nil {
			errs = append(errs, err)
		}
	}

	if gid, ok := d.GetOk("gid"); ok {
		err := ic.ExistGID(ctx, uint64(gid.(int)), c)
		if err != nil {
			errs = append(errs, err)
		}
	}

	return dc.ErrorsToDiagnostics(errs)
}

func checkParamsExistenceStaticRoute(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) diag.Diagnostics {
	vinsId := uint64(d.Get("vins_id").(int))

	if err := ic.ExistVins(ctx, vinsId, c); err != nil {
		return diag.FromErr(err)
	}

	return nil
}
